//
// Created by 阳坤 on 2022/3/1.
//

#include "utils/video_similarity.h"
#include "utils/count_down_latch.h"

extern "C" {
#include "utils/md5.h"
}

typedef struct VsimBean {
    const char *filepath;
    const char *image_md5;
    long start_time;
    long end_time;
    std::vector<VideoSimilarityModel *> lists;
    pthread_t id;
} VsimBean;

static void *run_thread(void *p) {
    auto *vsimBean = (VsimBean *) p;
    long id = video_similarity_detection_init(vsimBean->filepath);
    if (id > 0) {
        int ret = video_similarity_detection_start(id, 0, vsimBean->start_time, vsimBean->end_time, AUDIO, vsimBean->lists);
        char md5_str[64] = {0};
//        if (get_file_md5(vsimBean->filepath, md5_str) > 0);
//        vsimBean->image_md5 = strdup(md5_str);
        video_similarity_detection_close(id);
    }
    return 0;
}


/**
 * 1、判断比对文件的 MD5 是否相等
 *  1.1、如果相等，
 *   1.1.1、判断媒体数据 mate 信息是否一致，如果一致，判断前面 N 关键帧的指纹码
 *  1.2、如果不等
 *   1.2.1、判断媒体数据 mate 信息是否一致, 如果一致，判断前面 N 关键帧的指纹码
 * @param argc
 * @param argv
 * @return
 */
int main(int argc, char *argv[]) {
    int32_t size = 7;
    const char *filepath[size];
    //票圈视频iOS客户端上传
    filepath[0] = "http://rescdn.yishihui.com/longvideo/video/vpc/20220304/20820121YCuumFJkPsqfRo51So";
    //ios 原生上传
    filepath[1] = "http://rescdn.yishihui.com/longvideo/video/vpc/20220304/20820121sxPL3akcURUIHquge3";
    //iOS h5上传
    filepath[2] = "http://rescdn.yishihui.com/longvideo/video/vpc/20220304/20820121rm4gHAdXW1IBr1ASEM";
    //票圈视频安卓客户端上传
    filepath[3] = "http://rescdn.yishihui.com/longvideo/video/vpc/20220304/200908721BH3GFTuCyThA6LMBY";
    //安卓h5上传
    filepath[4] = "http://rescdn.yishihui.com/longvideo/video/vpc/20220304/20090872465XAee2WXc6Pjz55z";
    //安卓原生上传
    filepath[5] = "http://rescdn.yishihui.com/longvideo/video/vpc/20220304/20090872DaWq15NTiSgmJvMNwC";
//    filepath[5] = "http://rescdn.yishihui.com/longvideo/video/vpc/20220304/20090872TkYCoGR785GKZPPM7K";
    http://rescdn.yishihui.com/longvideo/video/vpc/20220304/17938576TxEh0UJXpzcoYRJuHu
    //本地 local 上传的视频
    filepath[6] = "/Users/devyk/Downloads/share_18b774aab823c23d8072d7c45f8b9636.MP4";

    VsimBean vsimBean[size];
    for (int i = 0; i < size; ++i) {
        vsimBean[i].filepath = strdup(filepath[i]);
        vsimBean[i].start_time = 0;
        vsimBean[i].end_time = -1;
        pthread_create(&vsimBean[i].id, 0, run_thread, &vsimBean[i]);
    }
    for (int i = 0; i < size; ++i) {
        pthread_join(vsimBean[i].id, 0);
        LOGE("################# path=%s size=%d \n", vsimBean[i].filepath, vsimBean[i].lists.size());
    }

    //开始比较
//    for (int i = 0; i < size; ++i) {
    for (int i = 0; i < 1; ++i) {
        auto a = vsimBean[6];
        for (int j = i; j < size - 1; ++j) {
            auto b = vsimBean[j];
            //比较 md5
//            if (md5_is_exactly_the_same(a.image_md5, b.image_md5) == 0) {
//                LOGE("path1=%s path2=%s MD5 完全相似\n", a.filepath, b.filepath);
//                continue;
//            } else {
//                LOGE("path1=%s path2=%s MD5 完全不相似\n", a.filepath, b.filepath);
//            }
            int minFrames = MIN(a.lists.size(), b.lists.size());
            int maxFrames = MAX(a.lists.size(), b.lists.size());
            int sim_frame_count = 0;
            //比较帧指纹
            for (int k = 0; k < minFrames; ++k) {
                float v_sim = fingerprint_compare(a.lists[k]->image_hash, b.lists[k]->image_hash,
                                                  MIN(a.lists[k]->img_len, b.lists[k]->img_len));
                if (v_sim > 0.90) {
                    sim_frame_count++;
                }
            }
            LOGE("path1=%s path2=%s 相似度=%f   \n", a.filepath, b.filepath,
                 (float) sim_frame_count / (float) maxFrames);
        }
    }

    LOGE("end \n");
    return 0;
}

